/* --COPYRIGHT--,BSD
 * Copyright (c) 2015, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/
//*****************************************************************************
// #includes
//*****************************************************************************
#include "grlib_thermostat.h"
#include "HAL_MSP_EXP430FR5969_Sharp96x96.h"
#include "Sharp96x96.h"
#include "msp430fr5969.h"

//*****************************************************************************
// SPI function definitions
//*****************************************************************************
void SPI_initMaster();
void SPI_enable();

#define SPI_SPEED	(SYSTEM_CLOCK_SPEED/2000000)

//*****************************************************************************
//
// Initializes the display driver.
//
// This function initializes the Sharp96x96 display. This function
// configures the GPIO pins used to control the LCD display when the basic
// GPIO interface is in use. On exit, the LCD has been reset and is ready to
// receive command and data writes.
//
// return None.
//
//*****************************************************************************
void HAL_LCD_initDisplay(void)
{
	    // Configure LCD_SCLK_PIN for SPI_CLK mode
		P1DIR |= BIT5;
		P1SEL1 |= BIT5;
		P1SEL0 &= ~BIT5;

	    // Configure LCD_MOSI_PIN for SPI_MOSI mode
		P2DIR |= BIT0;
		P2SEL1 |= BIT0;
		P2SEL0 &= ~BIT0;

	    // Provide power to LCD
		P4SEL0 &= ~BIT2;
		P4SEL1 &= ~BIT2;
		P4DIR |= BIT2;
		P4OUT |= BIT2;

		// Turn on DISP
		P4SEL0 &= ~BIT3;
		P4SEL1 &= ~BIT3;
		P4DIR |= BIT3;
		P4OUT |= BIT3;

		// Configure LCD_SPI_CS_PIN as output pin
		P2SEL0 &= ~BIT4;
		P2SEL1 &= ~BIT4;
		P2DIR |= BIT4;

		// Clear CS bit
		P2OUT &= ~BIT4;

		// Configure SPI peripheral in master mode
	    SPI_initMaster();
	    // Release SPI module from reset
	    SPI_enable();


}



//*****************************************************************************
//
// Writes command or data to the LCD Driver
//
// param ucCmdData is the 8 or 16 bit command to send to the LCD driver
// Uses the SET_LCD_DATA macro
//
// return None
//
//*****************************************************************************
void HAL_LCD_writeCommandOrData(uint16_t command)
{
	while (!(UCA0IFG & UCTXIFG));
	UCA0TXBUF = command;
}

//*****************************************************************************
//
// Clears CS line
//
// This macro allows to clear the Chip Select (CS) line
//
// return None
//
//*****************************************************************************
void HAL_LCD_clearCS(void){
	P2OUT &= ~BIT4;
}

//*****************************************************************************
//
// Set CS line
//
// This macro allows to set the Chip Select (CS) line
//
// return None
//
//*****************************************************************************
void HAL_LCD_setCS(void){
	P2OUT |= BIT4;
}

//*****************************************************************************
//
// Waits until the SPI communication with the LCD is finished a command to
// the LCD Driver
//
// param None
//
// return None
//*****************************************************************************
void HAL_LCD_waitUntilLcdWriteFinish(void)
{
	while  (UCA0STATW & UCBUSY);
}

//*****************************************************************************
//
// Disables Shapr96x96 LCD
//
// param None
//
// return None
//*****************************************************************************
void HAL_LCD_disableDisplay(void)
{
	P4OUT &= ~BIT2;
	P4OUT &= ~BIT3;
}

//*****************************************************************************
//
// Enables Shapr96x96 LCD
//
// param None
//
// return None
//*****************************************************************************
void HAL_LCD_enableDisplay(void)
{
	P4OUT |= BIT2;
	P4OUT |= BIT3;
}


//*****************************************************************************
//
// Prepare to write memory
//
// This macro unlocks flash memory controller and
// sets access right on flash controller
//
// return None
//
//*****************************************************************************
void HAL_LCD_prepareMemoryWrite()
{
}


//*****************************************************************************
//
// Finish memory writing
//
// This macro removes access rights on flash controller and
// locks flash memory controller.
//
// return None
//
//*****************************************************************************
void HAL_LCD_finishMemoryWrite()
{
	__no_operation();
}

//
// SPI functions
//

void SPI_initMaster()
{
    //Disable the USCI Module
    UCA0CTLW0 |= UCSWRST;

    //Reset OFS_UCAxCTLW0 values
    UCA0CTLW0 &= ~(UCCKPH + UCCKPL + UC7BIT + UCMSB +
                                              UCMST + UCMODE_3 + UCSYNC);

    //Reset OFS_UCAxCTLW0 values
    UCA0CTLW0 &= ~(UCSSEL_3);

    //Select Clock
    UCA0CTLW0 |= UCSSEL__SMCLK;

    UCA0BRW =(uint16_t)(SPI_SPEED);

    
    // Configure as SPI master mode.
    // Clock phase select, polarity, msb
    // UCMST = Master mode
    // UCSYNC = Synchronous mode
    // UCMODE_0 = 3-pin SPI     
    UCA0CTLW0 |= (UCMSB + UCCKPH + UCMST + UCSYNC + UCMODE_0);
}


void SPI_enable()
{
    //Reset the UCSWRST bit to enable the USCI Module
     UCA0CTLW0 &= ~(UCSWRST);
}
