/* --COPYRIGHT--,BSD
 * Copyright (c) 2015, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/
//*****************************************************************************
// #includes
//*****************************************************************************
#include "i2c.h"
#include "HDC1000.h"
#include "msp430fr5969.h"

//*****************************************************************************
// Local variables
//*****************************************************************************
uint8_t 	HDC_Data[4] = {0};					//Buffer for received bytes		

//*****************************************************************************
// HDC1000 function definitions
//*****************************************************************************

//*****************************************************************************
//
// Initializes pins for interfacing with HDC1000 and configure HDC1000 
//
//
// This function initializes a the HDC1000 interface and sends the configuration
// frame to the sensor
// 
// return None.
//
//*****************************************************************************
void HDC1000_init(void)
{
	 //Set nDRDY_HDC as input pin with interrupt capabilities - Pin 1.3
	  P1DIR &= ~BIT3;
	  // Internal pull-up on P1.3
	  P1OUT |= BIT3;
	  // Enable pull-up resistor
	  P1REN |= BIT3;
	  // P1.3 Hi/lo edge
	  P1IES |= BIT3;
	  // Clear all P1 interrupt flags                             
	  P1IFG = 0;    
	  // P1.3 interrupt enabled                            
	  P1IE |= BIT3; 

	  // I2C pins - SDA and SCL
	  P1DIR &= ~(BIT6 + BIT7);
	  P1SEL1 |= BIT6 + BIT7;                                              

	static const uint8_t HDC1000_CONFIGURATION_FRAME[3] =
	{
		//Configuration register address
		CONFIGURATION_ADDRESS,	
		//MSB of configuration (sets Temp & Humidity measurements, 11-bit resolution)
		TEMP_RH_11BIT_MSB,	
		//LSB of configuration	
		TEMP_RH_11BIT_LSB		
	};

		// Set TX pointer to HDC1000_CONFIGURATION_FRAME
    	p_i2c_transmitData = (uint8_t *)HDC1000_CONFIGURATION_FRAME;

    	//Load transmit byte counter
    	i2c_transmitCounter = sizeof HDC1000_CONFIGURATION_FRAME;

	 	// Ensure stop condition got sent
	    while (UCB0CTLW0 & UCTXSTP);           

		//Perform I2C write
		USCI_B_I2C_masterwrite(HDC1000_ADDRESS);
}

//*****************************************************************************
//
// Send sensor a message to sample temperature and humidity. 
//
//
// This function sends the HDC1000 the command to start measurement. HDC1000 will
// interrupt MSP (IO) when data is available for a read. 
// 
// return None.
//
//*****************************************************************************
void HDC1000_startMeasurement(void)
{	
	static const uint8_t HDC1000_START_MEAS_FRAME[1] =
	{
		TEMPERATURE_ADDRESS 	//Temp. register address
	};
	//Transmit array start address
	p_i2c_transmitData = (uint8_t *)HDC1000_START_MEAS_FRAME;
	//Load transmit byte counter
	i2c_transmitCounter = sizeof HDC1000_START_MEAS_FRAME;
	// Ensure stop condition got sent
    while (UCB0CTLW0 & UCTXSTP);            
	//Perform I2C write
	USCI_B_I2C_masterwrite(HDC1000_ADDRESS);

}

//*****************************************************************************
//
// Read temperature and humidity from sensor
//
//
// This function reads the data from the HDC1000. HDC1000_readMeasurement is called
// after the HDC1000 triggers a ready interrupt.
// 
// return None.
//
//*****************************************************************************
void HDC1000_readMeasurement(void)
{
	//Receive array start address
	p_i2c_receivedData = (uint8_t *)HDC_Data;
	//Read temp. & humidity from HDC1000
	USCI_B_I2C_masterReadByte(HDC1000_ADDRESS, sizeof HDC_Data);

	//Combine HDC Temp data bytes into one variable (allows for IQMath)
	HDC_Temp = (
			(0x00000000) |
			((uint32_t)HDC_Data[0] << 8) |
			((uint32_t)HDC_Data[1] << 0)
			);
	//Calculate temp according to HDC1000 datasheet
	HDC_Temp = ((HDC_Temp*165)>>16)-40;

	//Combine HDC RH data bytes into one 32-bit variable (allows for IQMath)
	HDC_RH = (
			(0x00000000) |
			((uint32_t)HDC_Data[2] << 8) |
			((uint32_t)HDC_Data[3] << 0)
			);

	//Calculate RH according to HDC1000 datasheet (leaves result in RH%)
	HDC_RH = (HDC_RH*100)>>16;
}


