/* --COPYRIGHT--,BSD
 * Copyright (c) 2015, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/
#include "grlib_thermostat.h"

//*****************************************************************************
//
// Initializes a drawing context.
//
// param context is a pointer to the drawing context to initialize.
// param display is a pointer to the Graphics_Display Info structure that
// describes the display driver to use.
//
// This function initializes a drawing context, preparing it for use.  The
// provided display driver will be used for all subsequent graphics
// operations, and the default clipping region will be set to the extent of
// the screen.
//
// return None.
//
//*****************************************************************************
void Graphics_initContext(Graphics_Context *context,
		const Graphics_Display *display)
{

    //
    // Set the size of the context.
    //
    context->size = sizeof(Graphics_Context);

    //
    // Save the pointer to the display.
    //
    context->display = display;

    //
    // Initialize the extent of the clipping region to the extents of the
    // screen.
    //
    context->clipRegion.xMin = 0;
    context->clipRegion.yMin = 0;
    context->clipRegion.xMax = display->width - 1;
    context->clipRegion.yMax = display->heigth - 1;

    //
    // Provide a default color and font.
    //
    context->foreground = 0;
    context->background = 0;
    context->font = 0;
}


//*****************************************************************************
//
// Sets the background color to be used.
//
// param context is a pointer to the drawing context to modify.
// param value is the 24-bit RGB color to be used.
//
// This function sets the background color to be used for drawing operations
// in the specified drawing context.
//
// return None.
//
//*****************************************************************************
void Graphics_setBackgroundColor(Graphics_Context *context, int32_t value)
{
	Graphics_Context *pC = context;
    pC->background = Graphics_translateColorOnDisplay(pC->display, value);
}


//*****************************************************************************
//
// Sets the font to be used.
//
// param context is a pointer to the drawing context to modify.
// param font is a pointer to the font to be used.
//
// This function sets the font to be used for string drawing operations in the
// specified drawing context.  If a tFontEx type font is to be used, cast its
// pointer to a font pointer before passing it as the font parameter.
//
// return None.
//
//*****************************************************************************
void Graphics_setFont(Graphics_Context *context, const Graphics_Font *font)
{
	Graphics_Context *pC = context;
	const Graphics_Font *pF = font;
	pC->font = pF;
}


//*****************************************************************************
//
// Sets the foreground color to be used.
//
// param context is a pointer to the drawing context to modify.
// param value is the 24-bit RGB color to be used.
//
// This function sets the color to be used for drawing operations in the
// specified drawing context.
//
// return None.
//
//*****************************************************************************
void Graphics_setForegroundColor(Graphics_Context *context, int32_t value)
{
	context->foreground = Graphics_translateColorOnDisplay(context->display,value);
}



//*****************************************************************************
//
// Flushes any cached drawing operations.
//
// param context is a pointer to the drawing context to use.
//
// This function flushes any cached drawing operations.  For display drivers
// that draw into a local frame buffer before writing to the actual display,
// calling this function will cause the display to be updated to match the
// contents of the local frame buffer.
//
// return None.
//
//*****************************************************************************
void Graphics_flushBuffer(const Graphics_Context *context)
{
	Graphics_flushOnDisplay(context->display);
}

//*****************************************************************************
//
// Forces a clear screen. Contents of Display buffer unmodified
//
// param context is a pointer to the drawing context to use.
//
// This function forces a clear screen.
//
// return None.
//
//*****************************************************************************
void Graphics_clearDisplay(const Graphics_Context *context)
{
	Graphics_clearDisplayOnDisplay(context->display,context->background );
}
//*****************************************************************************

