/* --COPYRIGHT--,BSD
 * Copyright (c) 2015, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/
#include <stdint.h>
 
//*****************************************************************************
// Syntax #defines
//*****************************************************************************
 #define bool   _Bool
 #define false  0
 #define true   1

#ifndef __GRLIB_THERMOSTAT_H__
#define __GRLIB_THERMOSTAT_H__


//*****************************************************************************
// Make sure min and max are defined.
//*****************************************************************************
#ifndef min
#define min(a, b)               (((a) < (b)) ? (a) : (b))
#endif

#ifndef max
#define max(a, b)               (((a) < (b)) ? (b) : (a))
#endif

//*****************************************************************************
// This structure defines the characteristics of a Bitmap Image
//*****************************************************************************
typedef struct Graphics_Image
{
    uint8_t bPP;                 //< Bits per pixel and Compressed/Uncompressed
    uint16_t xSize;              //< xSize
    uint16_t ySize;              //< ySize
    uint16_t numColors;          //< Number of Colors in Palette
    const uint32_t  * pPalette;  //< Pointer to Palette
    const uint8_t * pPixel;      //< Pointer to pixel data;
} Graphics_Image;

//*****************************************************************************
// This structure defines the extents of a rectangle.  All points greater than
// or equal to the minimum and less than or equal to the maximum are part of
// the rectangle.
//*****************************************************************************
typedef struct Graphics_Rectangle
{
    int16_t xMin;           //< The minimum X coordinate of the rectangle.
    int16_t yMin;           //< The minimum Y coordinate of the rectangle.
    int16_t xMax;           //< The maximum X coordinate of the rectangle.
    int16_t yMax;           //< The maximum Y coordinate of the rectangle.
} Graphics_Rectangle;


//*****************************************************************************
// This structure defines the characteristics of a display driver.
//*****************************************************************************
typedef struct Graphics_Display
{
    int32_t  size;              //< The size of this structure.
    void *displayData;          //< A pointer to display driver-specific data.
    uint16_t width;             //< The width of this display.
    uint16_t heigth;            //< The height of this display.
    void (*callPixelDraw)(void *displayData, int16_t x, int16_t y,
            uint16_t value);    //< A pointer to the function to draw a pixel on this display.
    void (*callPixelDrawMultiple)(void *displayData, int16_t x, int16_t y,
            int16_t x0, int16_t count, int16_t bPP, const uint8_t *data,
            const uint32_t *pucPalette);    //< A pointer to the function to draw multiple pixels on this display.
    void (*callLineDrawH)(void *displayData, int16_t x1, int16_t x2, int16_t y,
            uint16_t value);    //< A pointer to the function to draw a horizontal line on this display.
    void (*callLineDrawV)(void *displayData, int16_t x, int16_t y1,
            int16_t y2, uint16_t value); //< A pointer to the function to draw a vertical line on this display.
    void (*callRectFill)(void *displayData, const Graphics_Rectangle *rect,
            uint16_t value);    //< A pointer to the function to draw a filled rectangle on this display.
    uint32_t (*callColorTranslate)(void *displayData, uint32_t  value); //< A pointer to the function to translate 24-bit RGB colors to display-specific colors.
    void (*callFlush)(void *displayData); //< A pointer to the function to flush any cached drawing operations on this display.
    void (*callClearDisplay)(void *displayData, uint16_t value); //<  A pointer to the function to clears Display. Contents of display buffer unmodified
} Graphics_Display;

//*****************************************************************************
// This structure describes a font used for drawing text onto the screen.
//*****************************************************************************
typedef struct Graphics_Font
{
    uint8_t format;     //< The format of the font.  Can be one of FONT_FMT_UNCOMPRESSED or FONT_FMT_PIXEL_RLE.
    uint8_t maxWidth;   //< The maximum width of a character; this is the width of the widest character in the font, though any individual character may be narrower than this width.
    uint8_t height;     //< The height of the character cell; this may be taller than the font data for the characters (to provide inter-line spacing).
    uint8_t baseline;   //< The offset between the top of the character cell and the baseline of  the glyph.  The baseline is the bottom row of a capital letter, below which only the descenders of the lower case letters occur.
    uint16_t offset[96];//< The offset within data to the data for each character in the font.
    const uint8_t *data;//< A pointer to the data for the font.
} Graphics_Font;

//*****************************************************************************
// This is a newer version of the structure which describes a font used
// for drawing text onto the screen.  This variant allows a font to contain an
// arbitrary, contiguous block of codepoints from the 256 basic characters in
// an ISO8859-n font and allows support for accented characters in Western
// European languages and any left-to-right typeface supported by an ISO8859
// variant. Fonts encoded in this format may be used interchangeably with the
// original fonts merely by casting the structure pointer when calling any
// function or macro which expects a font pointer as a parameter.
//*****************************************************************************
typedef struct Graphics_FontEx
{
    uint8_t format;         //< The format of the font.  Can be one of FONT_FMT_EX_UNCOMPRESSED or FONT_FMT_EX_PIXEL_RLE.
    uint8_t maxWidth;       //< The maximum width of a character; this is the width of the widest character in the font, though any individual character may be narrower than this width.
    uint8_t height;         //< The height of the character cell; this may be taller than the font data for the characters (to provide inter-line spacing).
    uint8_t baseline;       //< The offset between the top of the character cell and the baseline of the glyph.  The baseline is the bottom row of a capital letter, below which only the descenders of the lower case letters occur.
    uint8_t first;      //< The codepoint number representing the first character encoded in the font.
    uint8_t last;           //< The codepoint number representing the last character encoded in the font.
    const uint16_t *offset; //< A pointer to a table containing the offset within data to the data for each character in the font.
    const uint8_t *data;    //< A pointer to the data for the font.
} Graphics_FontEx;

//*****************************************************************************
// This structure defines a drawing context to be used to draw onto the
// screen.  Multiple drawing contexts may exist at any time.
//*****************************************************************************
typedef struct Graphics_Context
{
    int32_t  size;                      //< The size of this structure.
    const Graphics_Display *display;    //< The screen onto which drawing operations are performed.
    Graphics_Rectangle clipRegion;      //< The clipping region to be used when drawing onto the screen.
    uint32_t  foreground;               //< The color used to draw primitives onto the screen.
    uint32_t  background;               //< The background color used to draw primitives onto the screen.
    const Graphics_Font *font;          //< The font used to render text onto the screen.
} Graphics_Context;

//*****************************************************************************
// Deprecated struct names.  These definitions ensure backwards compatibility
// but new code should avoid using deprecated struct names since these will
// be removed at some point in the future.
//*****************************************************************************
#define ClrWhite                                GRAPHICS_COLOR_WHITE
#define ClrBlack                                GRAPHICS_COLOR_BLACK
#define FONT_FMT_PIXEL_RLE                      GRAPHICS_FONT_FMT_PIXEL_RLE
#define tFontEx                                 Graphics_FontEx
#define tFont                                   Graphics_Font
#define tDisplay                                Graphics_Display
#define tRectangle                              Graphics_Rectangle
#define tImage                                  Graphics_Image
#define tContext                                Graphics_Context
#define sXMax                                   xMax
#define sXMin                                   xMin
#define sYMax                                   yMax
#define sYMin                                   yMin

//*****************************************************************************
// Deprecated function names.  These definitions ensure backwards compatibility
// but new code should avoid using deprecated function names since these will
// be removed at some point in the future.
//*****************************************************************************
#define GrContextInit                       Graphics_initContext
#define GrStringDraw                        Graphics_drawString
#define GrContextBackgroundSet              Graphics_setBackgroundColor
#define GrContextFontSet                    Graphics_setFont
#define GrContextForegroundSet              Graphics_setForegroundColor
#define GrFlush                             Graphics_flushBuffer
#define GrClearDisplay                      Graphics_clearDisplay

//*****************************************************************************
// Indicates that the font data is stored in an uncompressed format.
//*****************************************************************************
#define GRAPHICS_FONT_FMT_UNCOMPRESSED   0x00

//*****************************************************************************
// Indicates that the font data is stored using a pixel-based RLE format.
//*****************************************************************************
#define GRAPHICS_FONT_FMT_PIXEL_RLE      0x01

//*****************************************************************************
// A marker used in the format field of a font to indicates that the font
// data is stored using the new tFontEx structure.
//*****************************************************************************
#define GRAPHICS_FONT_EX_MARKER          0x80

//*****************************************************************************
// Indicates that the font data is stored in an uncompressed format and uses
// the tFontEx structure format.
//*****************************************************************************
#define GRAPHICS_FONT_FMT_EX_UNCOMPRESSED   (FONT_FMT_UNCOMPRESSED | FONT_EX_MARKER)

//*****************************************************************************
// Indicates that the font data is stored using a pixel-based RLE format and
// uses the tFontEx structure format.
//*****************************************************************************
#define GRAPHICS_FONT_FMT_EX_PIXEL_RLE      (FONT_FMT_PIXEL_RLE | FONT_EX_MARKER)

//*****************************************************************************
// Value to automatically draw the entire length of the string
// (subject to clipping)
//*****************************************************************************
#define GRAPHICS_AUTO_STRING_LENGTH     -1


//*****************************************************************************
// A set of color definitions.  This set is the subset of the X11 colors (from
// rgb.txt) that are supported by typical web browsers.
//*****************************************************************************
#define GRAPHICS_COLOR_WHITE                     0x00FFFFFF
#define GRAPHICS_COLOR_BLACK                     0x00000000



//*****************************************************************************
// Prototypes for the predefined fonts in the graphics library.  ..Cm.. is the
// computer modern font, which is a serif font.  ..Cmsc.. is the computer
// modern small-caps font, which is also a serif font.  ..Cmss.. is the
// computer modern sans-serif font.
//*****************************************************************************
extern const Graphics_Font g_sFontCmss12;
extern const Graphics_Font g_sFontCmss20b;

//*****************************************************************************
// Prototypes from graphics library functions.
//*****************************************************************************

extern void Graphics_initContext(Graphics_Context *context,
        const Graphics_Display *display);
extern void Graphics_drawLineH(const Graphics_Context *context, int32_t  x1,
        int32_t  x2, int32_t  y);
extern void Graphics_drawString(const Graphics_Context *context, int8_t *string,
       int32_t  lLength, int32_t  x, int32_t  y, bool  opaque);
extern void Graphics_setBackgroundColor(Graphics_Context *context,
        int32_t value);
extern void Graphics_setFont(Graphics_Context *context,
        const Graphics_Font *font);
extern void Graphics_setForegroundColor(Graphics_Context *context,
        int32_t value);
extern uint32_t Graphics_translateColorOnDisplay(const Graphics_Display *display,
        uint32_t value);
extern void Graphics_drawHorizontalLineOnDisplay(
        const Graphics_Display *display, uint16_t x1, uint16_t  x2, uint16_t  y,
        uint32_t value);
extern void Graphics_flushOnDisplay(const Graphics_Display *display);
extern void Graphics_clearDisplay(const Graphics_Context *context);
extern void Graphics_flushBuffer(const Graphics_Context *context);
extern void Graphics_drawPixelOnDisplay(const Graphics_Display *display,
        uint16_t x, uint16_t y, uint16_t value);
extern void Graphics_clearDisplayOnDisplay(const Graphics_Display *display,
        uint16_t value);



#endif // __GRLIB_THERMOSTAT_H__
