/* --COPYRIGHT--,BSD
 * Copyright (c) 2015, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/
//*****************************************************************************
// #includes
//*****************************************************************************
#include "i2c.h"
#include "msp430fr5969.h"

//*****************************************************************************
// I2C transmission variables
//*****************************************************************************
uint8_t 	i2c_transmitCounter = 0;	//Variable to store transmit status for I2C
uint8_t 	*p_i2c_transmitData;		//Pointer to I2C transmit data
uint8_t 	i2c_receiveCounter = 0;		//Variable to store receive status for I2C
uint8_t 	*p_i2c_receivedData;		//Pointer to I2C received data
uint16_t 	i2c_mode = 0;				//Variable to store i2c mode (tx or rx)


//*****************************************************************************
// I2C function definitions
//*****************************************************************************

//*****************************************************************************
//
// Initializes an I2C communication channel: 400kHz, Master mode, sync, SMCLK
//
//
// This function initializes a the I2C interface including IO and peripheral 
// configuration.
// 
// return None.
//
//*****************************************************************************
void USCI_B_I2C_masterInit(void)
{
	  // Disable the GPIO power-on default high-impedance mode to activate
	  // previously configured port settings
	  PM5CTL0 &= ~LOCKLPM5;

	  // Software reset enabled
	  UCB0CTLW0 = UCSWRST;        
	  // I2C mode, Master mode, sync, SMCLK             
	  UCB0CTLW0 |= UCMODE_3 | UCMST | UCSYNC | UCSSEL__SMCLK; 
	  // Slave address is 7 bits 
	  UCB0CTLW0 &= ~(UCSLA10); 
	  // Set own address to 0x5D                 
	  UCB0I2COA0 = 0x5D;	
	  // baudrate = SMCLK / 40 = 400kHz					
	  UCB0BRW = 0x0028;       
	  // Release device from reset to operate                  
	  UCB0CTL1 &= ~UCSWRST;					

}

//*****************************************************************************
//
// USCI_B_I2C_masterwrite
//
//
// This function performs an I2C write. Function ensure correct configuration
// to write data including interrupt, delays and data pointer handling
// 
// param SLAVE_ADDRESS is an integer that sets the slave address of the sensor
//
// return None.
//
//*****************************************************************************
void USCI_B_I2C_masterwrite(uint8_t SLAVE_ADDRESS)
{	
	// Ensure mode is set to trasmit
	i2c_mode = (UCB0CTLW0 & UCTR);

	if(i2c_mode != UCTR)
	{
		UCB0CTLW0 |= (UCTR);
	}

	//Delay until I2C module is free
	while(UCB0STAT & UCBBUSY);

	//Specify slave address
	UCB0I2CSA = SLAVE_ADDRESS;

	//Enable I2C Module to start operations
	UCB0CTLW0 &= ~UCSWRST;

	//Clear transmit Interrupt
	UCB0IFG &= ~(UCTXIFG1);

	//Enable transmit Interrupt
	UCB0IE = UCTXIE0;             // transmit and NACK interrupt enable

	//Delay between each transaction
	__delay_cycles(50);

	//Initiate start and send first character
	USCI_B_I2C_masterMultiByteSendStart(*p_i2c_transmitData++);

	// Decrement transmit counter
	i2c_transmitCounter--;

	//Enter LPM0 with interrupts enabled
	__bis_SR_register( LPM0_bits + GIE );

}


//*****************************************************************************
//
// I2C read byte
//
//
// This function performs an I2C read. Function ensure correct configuration
// to read data including interrupt, delays and data pointer handling
// 
// param SLAVE_ADDRESS is an integer that sets the slave address of the sensor
//
// return None.
//
//*****************************************************************************
void USCI_B_I2C_masterReadByte(uint8_t SLAVE_ADDRESS, uint8_t BYTE_COUNT)
{
	
	// Ensure mode is set to receive
	i2c_mode = (UCB0CTLW0 & UCTR);

	if(i2c_mode != 0x00)
	{
		UCB0CTLW0 &= ~(UCTR);
	}

	i2c_receiveCounter = BYTE_COUNT;

	//Delay until I2C module is free
	while(UCB0STAT & UCBBUSY);


	//Set slave address
	UCB0I2CSA = SLAVE_ADDRESS;
	//Enable I2C Module to start operations
	UCB0CTLW0 &= ~(UCSWRST);


	//Clear receive interrupt
	 UCB0IFG &= ~(UCRXIFG1);

	 // Enable receive interrupt
	 UCB0IE = UCRXIE;
	__delay_cycles(50);

	if(BYTE_COUNT == 1)
	{
		//Initiate I2C receive
	    //local variable to store GIE status
	    uint16_t gieStatus;

	    //Store current SR register
	    gieStatus = __get_SR_register() & GIE;

	    //Disable global interrupt
	    __disable_interrupt();

	    //Set USCI in Receive mode
	    UCB0CTLW0 &= ~UCTR;

	    //Send start condition.
	    UCB0CTLW0 |= UCTXSTT;

	    //Poll for Start bit to complete
	    while (UCB0CTLW0 & UCTXSTT) ;

	    //Send stop condition.
	    UCB0CTLW0 |= UCTXSTP;

	    //Reinstate SR register
	    __bis_SR_register(gieStatus);
	}
	else
	{
		//Initiate I2C receive
	    //Set USCI in Receive mode
		UCB0CTLW0 &= ~UCTR;
	    //Send start
	    UCB0CTLW0 |= UCTXSTT;
	}

	//Enter LPM0 with interrupts enabled
	__bis_SR_register( LPM0_bits + GIE );


}

//*****************************************************************************
//
// I2C send start and load first data byte into TX buffer
//
//
// This function starts an I2C write. Function ensure start condition is sent 
// and then does a write of the first data byte
// 
// param txData is the first byte to be transmitted
//
// return None.
//
//*****************************************************************************
void USCI_B_I2C_masterMultiByteSendStart (uint8_t txData)
{
    //Store current transmit interrupt enable
    uint8_t txieStatus = UCB0IE & UCTXIE;

    //Disable transmit interrupt enable
    UCB0IE &= ~(UCTXIE);

    //Send start condition.
    UCB0CTL1 |= UCTR +  UCTXSTT;

    //Poll for transmit interrupt flag.
    while (!(UCB0IFG & UCTXIFG)) ;

    //Send single byte data.
    UCB0TXBUF = txData;

    //Reinstate transmit interrupt enable
    UCB0IE |= txieStatus;
}



//*****************************************************************************
// Interrupt Service Routine - I2C
//*****************************************************************************
#pragma vector = USCI_B0_VECTOR
__interrupt void USCI_B0_ISR(void)
{
  switch(__even_in_range(UCB0IV, USCI_I2C_UCBIT9IFG))
  {

    case USCI_NONE:          break;         // Vector 0: No interrupts
    case USCI_I2C_UCALIFG:   break;         // Vector 2: ALIFG
    case USCI_I2C_UCNACKIFG:                // Vector 4: NACKIFG
     	 UCB0CTLW0 |= UCTXSTT;                 // resend start if NACK
      	 break;

    case USCI_I2C_UCSTTIFG:  break;         // Vector 6: STTIFG
    case USCI_I2C_UCSTPIFG:  break;         // Vector 8: STPIFG
    case USCI_I2C_UCRXIFG3:  break;         // Vector 10: RXIFG3
    case USCI_I2C_UCTXIFG3:  break;         // Vector 12: TXIFG3
    case USCI_I2C_UCRXIFG2:  break;         // Vector 14: RXIFG2
    case USCI_I2C_UCTXIFG2:  break;         // Vector 16: TXIFG2
    case USCI_I2C_UCRXIFG1:  break;         // Vector 18: RXIFG1
    case USCI_I2C_UCTXIFG1:  break;         // Vector 20: TXIFG1
    case USCI_I2C_UCRXIFG0:                 // Vector 22: RXIFG0    	
       	//Decrement RX byte counter
    	i2c_receiveCounter--;
    	if(i2c_receiveCounter)
    	{
    		if(i2c_receiveCounter == 1)
    		{
    			//Initiate end of reception -> Receive byte with NAK
    		    //Send stop condition.
    			UCB0CTL1 |= UCTXSTP;
    		    //Capture data from receive buffer after setting stop bit due to
    		    //MSP430 I2C critical timing.
    			*p_i2c_receivedData++ = UCB0RXBUF;
    		     // Ensure stop condition got sent
    			while (UCB0CTL1 & UCTXSTP);            
    		    //Wait for RX buffer
    			while (!(UCB0IFG & UCRXIFG));
    		}
    		else
    		{
    			//Keep receiving one byte at a time
    			*p_i2c_receivedData++ = UCB0RXBUF;
    		}
    	}
    	else
    	{
    		//Receive single byte
    	    //Polling RXIFG0 if RXIE is not enabled
    	    if(!(UCB0IE & UCRXIE)) {
    	        while(!(UCB0IFG & UCRXIFG));
    	    }
    		*p_i2c_receivedData = UCB0RXBUF;
    		// Exit in active mode
    		__bic_SR_register_on_exit(LPM0_bits);
   		}	
    	break;
    
    case USCI_I2C_UCTXIFG0:                 // Vector 24: TXIFG0
		//Check TX byte counter
		if (i2c_transmitCounter)
		{
			//Initiate send of character from Master to Slave
			UCB0TXBUF = *p_i2c_transmitData++;
			//Decrement TX byte counter
			i2c_transmitCounter--;
		}
		else
		{
			// I2C stop condition
			UCB0CTLW0 |= UCTXSTP;               
			// Clear USCI_B0 TX int flag
			UCB0IFG &= ~UCTXIFG;                
			// Exit in active mode
			__bic_SR_register_on_exit(LPM0_bits); 
		}
      break;

    default: break;


  }
}
